/********************************************************************************
*	NOTICE:														*
*																*
*	This code is believed by the author to be bug free.  You are free to use	*
*	and modify this code with the understanding that any use, either direct	*
*	or derivative, must contain acknowledgement of its author and source.		*
*	The author makes no warranty of any kind, expressed or implied, of		*
*	merchantability or fitness for a particular purpose.  The author shall	*
*	not be held liable for any incidental or consequential damages in		*
*	connection with or arising out of the furnishing, performance, or use of	*
*	this software.  This software is not authorized for use in life support	*
*	devices or systems.												*
*																*
********************************************************************************/

// AIPP software developed by Gianluca Di Pasqua, John Oh, and Sean Hanley
// in association with the DSP Lab at the University of Toronto, 2000.

import java.awt.*;
import java.awt.image.*;
import java.awt.MediaTracker;
import java.io.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
 

public class AIPPhandler implements ItemListener,ActionListener
{
	int width;
	int height;
	int pix[];
	byte info[];
	private FileInputStream file_open;
	private FileWriter file_save;
	
	String loadFilename;
	String loadFileDir;
	String temp;
		
	AIPP myframe;
	Image pic;
	Image picold;

	FileDialog openFileDialog;
	FileDialog saveFileDialog;
	
	static int masksize;
	static float variance;
	static float edge_thresh;
	static double hsi1,hsi2,hsi3,hsi4,hsi5;
	static boolean ok;
	
	boolean custom;
	
	
	AIPPhandler(AIPP win, String t1, String t2)
	{
		myframe=win;
		openFileDialog=new FileDialog(myframe,"Open Image",FileDialog.LOAD);
		saveFileDialog=new FileDialog(myframe,"Save Image",FileDialog.SAVE);
		// set you own image directory here!!!
		openFileDialog.setDirectory("./Images/");		
		saveFileDialog.setDirectory("./Images/");

		loadFilename=t1;
		loadFileDir=t2;

		masksize=3;
		picold = myframe.image;
		pic = myframe.image;
		ok = false;
	}
	
	public void eventStart()
	{
		myframe.mainCanvas.setCursor(AIPP.waitCursor);
		myframe.openm.setEnabled(false);
		myframe.savem.setEnabled(false);
		myframe.editM.setEnabled(false);
		myframe.filtM.setEnabled(false);
		myframe.analM.setEnabled(false);
		myframe.noiseM.setEnabled(false);
		AIPP.statusBar.setText("AIPP Working ...");
	}
	
	public void eventEnd(int type)
	{
		MediaTracker mt=new MediaTracker(myframe);
		mt.addImage(pic,0);
		try
		{
			mt.waitForAll();
		}
		catch (Exception ex)
		{
		}

		myframe.mainCanvas.setCursor(AIPP.defaultCursor);
		myframe.openm.setEnabled(true);
		myframe.savem.setEnabled(true);
		myframe.editM.setEnabled(true);
		myframe.filtM.setEnabled(true);
		myframe.analM.setEnabled(true);
		myframe.noiseM.setEnabled(true);
		if (type == 1)
		{
			// event was successful
			// set old image and current image 
			// rebuild current image array ro reflect changes
			picold=myframe.image;
			myframe.image=pic;
			myframe.getArray(pic);
		}
		System.gc();
	}
		
	private void open_ppm(int type)
	{
		if(loadFilename != null)
		{
			int index=0;
			int colors=3;
			info=new byte[100];		

			try
			{
				file_open=new FileInputStream(loadFileDir+loadFilename);
				file_open.read(info);
						
				int flag=0;
				int header=0;
				int temp=0;
				int valid=0;
				String width_ascii="";
				String height_ascii="";

				// Parse the ppm header, get width and height
				while(flag == 0)
				{
					temp=info[header++];

					if (temp == 35)
						valid=0;
					else if (temp == 10 && valid == 0)
						valid=1;
					else if (temp == 32 && valid == 1)
						valid=2;
					else if (temp == 10 && valid == 2)
						valid=3;
					else if (temp == 10 && valid == 3)
						flag=1;

					// read width and height and build string	
					if (valid == 1 && temp != 35 && temp != 10)
					{
						width_ascii += String.valueOf((char)temp);
					}
					else if (valid == 2 && temp != 32 && temp != 10)
					{
						height_ascii += String.valueOf((char)temp);
					}							
				}
				
				// check second character if 5 (for P5) or 6 (for P6)
				// if it is not enforce correct type in case of user error
				// saved file with wrong extension
				if (info[1] == 53)
					type=2;
				else if (info[1] == 54)
					type=1;
						
				// convert string into int
				width=Integer.parseInt(width_ascii);
				height=Integer.parseInt(height_ascii);
				file_open.close();
				file_open=new FileInputStream(loadFileDir+loadFilename);
				if (type == 1)
					info=new byte[width*height*colors+header];
				else if (type ==2)
					info=new byte[width*height+header];
				
				file_open.read(info);
				file_open.read(info);
				pix=new int[width*height];
				index=0;
				
				if (type == 1)
				{
					//interleaved rgb ppm model 24-bit
					for (int y=header;y<colors*height*width;)
					{
						pix[index++]=(255<<24)|(info[y++]&0xff)<<16|(info[y++]&0xff)<<8|info[y++]&0xff;
					}
				}
				else if (type == 2)
				{
					//interleaved rgb pgm model (greyscale of ppm)
					for (int y=header;y<height*width;)
					{
						pix[index++]=(255<<24)|(info[y]&0xff)<<16|(info[y]&0xff)<<8|info[y++]&0xff;
					}
				}
				
				file_open.close();
			}
			catch (IOException Ie)
			{
			}
		}
	
	}

	public void actionPerformed(ActionEvent e)
	{
		pic=myframe.image;
		
		if (e.getSource() instanceof MenuItem)
		{
			String label=((MenuItem)e.getSource()).getLabel();
			if (label.equals("Open") || label.equals("Revert to Original"))
			{
				eventStart();
				if (myframe.infom.getState() == true)
				{
					myframe.infoBox = false;
					myframe.infoPopup.dispose();
					myframe.infom.setState(false);
				}
				
				if (label.equals("Open"))
				{
					openFileDialog.setVisible(true);
					loadFilename=openFileDialog.getFile();
					loadFileDir=openFileDialog.getDirectory();
				}

				if (loadFilename != null)
				{
					temp=loadFilename;
					temp=temp.toLowerCase();
				
					// check extension
					// pgm is same as ppm but gresycale image with header P5
					if (temp.endsWith(".ppm") == true || temp.endsWith(".pgm") == true)
					{
						if (temp.endsWith(".ppm") == true) open_ppm(1);
						else open_ppm(2);
							
						pic=myframe.createImage(new MemoryImageSource(width,height,pix,0,width));

						MediaTracker mt=new MediaTracker(myframe);
						mt.addImage(pic,0);
						try
						{
							mt.waitForAll();
						}
						catch (Exception ex)
						{
						}
						myframe.getArray(pic,width,height);
						eventEnd(1);
						AIPP.statusBar.setText("Opened image: "+loadFilename);
					}
					else if (temp.endsWith(".gif") == true || temp.endsWith(".jpg") == true
						|| temp.endsWith(".jpeg") == true)
					{
						pic=Toolkit.getDefaultToolkit().getImage(loadFileDir+loadFilename);				
						eventEnd(1);
						AIPP.statusBar.setText("Opened image: "+loadFilename);
					}
					else
					{
						AIPP.statusBar.setText("Cannot open file: "+loadFilename);
						eventEnd(0);				
					}
				}
				else
				{
					AIPP.statusBar.setText("Operation Canceled");
					eventEnd(0);
				}
			}
			else if (label.equals("Save"))
			{
//				AIPP.statusBar.setText("Save function not supported in AIPP Applet");
				eventStart();
				saveFileDialog.setVisible(true);
				loadFilename = saveFileDialog.getFile();
				loadFileDir = saveFileDialog.getDirectory();
				
				if (loadFilename != null)
				{
					// add extension
					if (loadFilename.toLowerCase().endsWith(".ppm") == false
						&& loadFilename.toLowerCase().endsWith(".pgm") == false)
					{
						loadFilename = loadFilename.concat(".ppm");
					}

					save_ppm filter = new save_ppm(loadFileDir,loadFilename);
					pic = myframe.createImage(new FilteredImageSource(myframe.image.getSource(),filter));
					eventEnd(1);
				}
				else
				{
					AIPP.statusBar.setText("Operation Canceled");
					eventEnd(0);
				}
			}
			else if (label.equals("Undo"))
			{
				pic=myframe.image;
				myframe.image=picold;
				picold=pic;
				AIPP.statusBar.setText("Undo Performed");
			}
			else if (label.equals("Exit"))
			{
				myframe.dispose();
				System.exit(0);
			}
			else if (label.equals("Grey"))
			{
				eventStart();
				greyfilt filter=new greyfilt();
				pic=myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Image converted to Grey-Scale");
			}
			else if (label.equals("Negative"))
			{
				eventStart();
				negfilt filter=new negfilt();
				pic=myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Negative Filter Applied");
			}
			else if (label.equals("Emboss"))
			{
				eventStart();
				int mask[] = {-2,-1,0,-1,0,1,0,1,2};
				embfilt filter=new embfilt(3,mask);
				pic=myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Emboss Filter Applied");
			}
			else if (label.equals("Blue"))
			{
				eventStart();
				bluefilt filter=new bluefilt();
				pic=myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Blue Channel");
			}
			else if (label.equals("Green"))
			{
				eventStart();
				greenfilt filter=new greenfilt();
				pic=myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Green Channel");
			}
			else if (label.equals("Red"))
			{
				eventStart();
				redfilt filter=new redfilt();
				pic=myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Red Channel");
			}
			else if (label.equals("Blur"))
			{
				eventStart();
				if (custom)
				{
					int mask[] = new int[masksize*masksize];
					genDialog gd = new genDialog(myframe,mask);
					gd.setVisible(true);
					AIPP.statusBar.setText("AIPP Working ...");

					if (ok)
					{
						blurfilt filter=new blurfilt(masksize,mask);
						pic=myframe.createImage(new FilteredImageSource
							(myframe.image.getSource(),filter));
						eventEnd(1);
						AIPP.statusBar.setText("Blur Filter Applied");
					}
					else
					{
						eventEnd(0);
						AIPP.statusBar.setText("Operation Canceled");
					}
				}
				else
				{
					int mask[] = new int[masksize*masksize];
				
					for (int x=0;x<masksize*masksize;x++)
						mask[x]=1;

					blurfilt filter=new blurfilt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Blur Filter Applied");
				}
			}
			else if (label.equals("Sharpen"))
			{
				eventStart();
				if (custom)
				{
					int mask[] = new int[masksize*masksize];
					genDialog gd = new genDialog(myframe,mask);
					gd.setVisible(true);
					AIPP.statusBar.setText("AIPP Working ...");

					if (ok)
					{
						sharpfilt filter=new sharpfilt(masksize,mask);
						pic=myframe.createImage(new FilteredImageSource
							(myframe.image.getSource(),filter));
						eventEnd(1);
						AIPP.statusBar.setText("Sharpen Filter Applied");
					}
					else
					{
						eventEnd(0);
						AIPP.statusBar.setText("Operation Canceled");
					}
				}
				else
				{
					int mask[] =new int[masksize*masksize];
				
					for (int x=0;x<masksize*masksize;x++)
						mask[x]=1;

					sharpfilt filter=new sharpfilt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Sharpen Filter Applied");
				}
			}
			else if (label.equals("D_Plus"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	0,1,0,
							1,1,1,
							0,1,0};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	0,0,1,0,0,
							0,0,1,0,0,
							1,1,1,1,1,
							0,0,1,0,0,
							0,0,1,0,0};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	0,0,0,1,0,0,0,
							0,0,0,1,0,0,0,
							0,0,0,1,0,0,0,
							1,1,1,1,1,1,1,
							0,0,0,1,0,0,0,
							0,0,0,1,0,0,0,
							0,0,0,1,0,0,0};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Dilate Filter(D-Plus) Applied");
			}
			else if (label.equals("D_Square"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	1,1,1,
							1,0,1,
							1,1,1};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	1,1,1,1,1,
							1,0,0,0,1,
							1,0,0,0,1,
							1,0,0,0,1,
							1,1,1,1,1};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	1,1,1,1,1,1,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,1,1,1,1,1,1};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Dilate Filter(D-Square) Applied");
			}
			else if (label.equals("D_X"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	1,0,1,
							0,1,0,
							1,0,1};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	1,0,0,0,1,
							0,1,0,1,0,
							0,0,1,0,0,
							0,1,0,1,0,
							1,0,0,0,1};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	1,0,0,0,0,0,1,
							0,1,0,0,0,1,0,
							0,0,1,0,1,0,0,
							0,0,0,1,0,0,0,
							0,0,1,0,1,0,0,
							0,1,0,0,0,1,0,
							1,0,0,0,0,0,1};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Dilate Filter(D-X) Applied");
			}
			else if (label.equals("D_Circle"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	0,1,0,
							1,0,1,
							0,1,0};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	0,0,1,0,0,
							0,1,0,1,0,
							1,0,0,0,1,
							0,1,0,1,0,
							0,0,1,0,0};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	0,0,1,1,1,0,0,
							0,1,0,0,0,1,0,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							0,1,0,0,0,1,0,
							0,0,1,1,1,0,0};
					dilate_filt filter=new dilate_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Dilate Filter(D-Circle) Applied");
			}
			else if (label.equals("E_Plus"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	0,1,0,
							1,1,1,
							0,1,0};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	0,0,1,0,0,
							0,0,1,0,0,
							1,1,1,1,1,
							0,0,1,0,0,
							0,0,1,0,0};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	0,0,0,1,0,0,0,
							0,0,0,1,0,0,0,
							0,0,0,1,0,0,0,
							1,1,1,1,1,1,1,
							0,0,0,1,0,0,0,
							0,0,0,1,0,0,0,
							0,0,0,1,0,0,0};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Erode Filter(E-Plus) Applied");
			}
			else if (label.equals("E_Square"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	1,1,1,
							1,0,1,
							1,1,1};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	1,1,1,1,1,
							1,0,0,0,1,
							1,0,0,0,1,
							1,0,0,0,1,
							1,1,1,1,1};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	1,1,1,1,1,1,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,1,1,1,1,1,1};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Erode Filter(E-Square) Applied");
			}
			else if (label.equals("E_X"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	1,0,1,
							0,1,0,
							1,0,1};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	1,0,0,0,1,
							0,1,0,1,0,
							0,0,1,0,0,
							0,1,0,1,0,
							1,0,0,0,1};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	1,0,0,0,0,0,1,
							0,1,0,0,0,1,0,
							0,0,1,0,1,0,0,
							0,0,0,1,0,0,0,
							0,0,1,0,1,0,0,
							0,1,0,0,0,1,0,
							1,0,0,0,0,0,1};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Erode Filter(E-X) Applied");
			}
			else if (label.equals("E_Circle"))
			{
				eventStart();

				if (masksize == 3)
				{
					int mask[] = {	0,1,0,
							1,0,1,
							0,1,0};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 5)
				{
					int mask[] = {	0,0,1,0,0,
							0,1,0,1,0,
							1,0,0,0,1,
							0,1,0,1,0,
							0,0,1,0,0};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				else if (masksize == 7)
				{
					int mask[] = {	0,0,1,1,1,0,0,
							0,1,0,0,0,1,0,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							1,0,0,0,0,0,1,
							0,1,0,0,0,1,0,
							0,0,1,1,1,0,0};
					erode_filt filter=new erode_filt(masksize,mask);
					pic=myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
				}
				eventEnd(1);
				AIPP.statusBar.setText("Erode Filter(E-Circle) Applied");
			}
			else if (label.equals("Fuzzy_Vector_Directional"))
			{
				eventStart();
				FVDF filter = new FVDF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Fuzzy_Vector_Directional Applied");
			}
			else if (label.equals("Vector_Median"))
			{
				eventStart();
				VMF filter = new VMF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Vector_Median Applied");
			}
			else if (label.equals("Arithmatic_Mean"))
			{
				eventStart();
				AMF filter = new AMF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Arithmatic_Mean Applied");
			}
			else if (label.equals("Basic_Vector_Directional"))
			{
				eventStart();
				BVDF filter = new BVDF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Basic_Vector_Directional Applied");
			}
			else if (label.equals("Generalized_Vector_Directional"))
			{
				eventStart();
				GVDF filter = new GVDF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Generalized_Vector_Directional Applied");
			}
			else if (label.equals("Directional_Distance"))
			{
				eventStart();
				DDF filter = new DDF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Directional_Distance Applied");
			}
			else if (label.equals("Hybrid_Directional"))
			{
				eventStart();
				HF filter = new HF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Hybrid_Directional Applied");
			}
			else if (label.equals("Adaptive_Hybrid_Directional"))
			{
				eventStart();
				AHF filter = new AHF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_Hybrid_Directional Applied");
			}
			else if (label.equals("Adaptive_Nearest_Neighbor"))
			{
				eventStart();
				ANNF filter = new ANNF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_Nearest_Neighbor Applied");
			}
			else if (label.equals("Adaptive_Nearest_Neighbor(Canberra)"))
			{
				eventStart();
				CANNF filter = new CANNF(3);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_Nearest_Neighbor(Canberra) Applied");
			}
			else if (label.equals("Adaptive_Nearest_Neighbor(S)"))
			{
				eventStart();
				CBANNF filter = new CBANNF(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_Nearest_Neighbor Applied");
			}
			else if (label.equals("Adaptive_NonParametric(Exponential)"))
			{
				eventStart();
				AMNFE filter = new AMNFE(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_NonParametric Applied");
			}
			else if (label.equals("Adaptive_NonParametric(Gaussian)"))
			{
				eventStart();
				AMNFG filter = new AMNFG(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_NonParametric Applied");
			}
			else if (label.equals("Adaptive_NonParametric(Directional)"))
			{
				eventStart();
				AMNFD filter = new AMNFD(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Adaptive_NonParametric Applied");
			}			
			else if (label.equals("Bayesian Adaptive(Median/Mean)"))
			{
				eventStart();
				BFMA filter = new BFMA(masksize);
				pic = myframe.createImage(new FilteredImageSource
					(myframe.image.getSource(),filter));
				eventEnd(1);
				AIPP.statusBar.setText("Bayesian Adaptive(Median/Mean) Applied");
			}			
			else if (label.equals("HSI Segmentation"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,3);
				gd.setVisible(true);
				if (ok)
				{
					hsi filter = new hsi(hsi1,hsi2,hsi3,hsi4,hsi5);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Mean"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,2);
				gd.setVisible(true);
				if (ok)
				{
					mean_edge filter = new mean_edge(edge_thresh,masksize);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Edge Detection (Mean) Completed");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Median"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,2);
				gd.setVisible(true);
				if (ok)
				{
					median_edge filter = new median_edge(edge_thresh,masksize);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Edge Detection (Median) Completed");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("VR (Euclidean Distance)"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,2);
				gd.setVisible(true);
				if (ok)
				{
					vr0_edge filter = new vr0_edge(edge_thresh,masksize);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Edge Detection (VR-Euclidean) Completed");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("VR (Angle Distance)"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,2);
				gd.setVisible(true);
				if (ok)
				{
					vr3_edge filter = new vr3_edge(edge_thresh,masksize);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Edge Detection (VR-Angle) Completed");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Adaptive"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,2);
				gd.setVisible(true);
				if (ok)
				{
					adapt_edge filter = new adapt_edge(edge_thresh,masksize);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Edge Detection (Adaptive) Completed");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Gaussian"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,1);
				gd.setVisible(true);
				if (ok)
				{
					gaus_noise filter = new gaus_noise(variance);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Gaussian Noise Added");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Impulsive"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,1);
				gd.setVisible(true);
				if (ok)
				{
					imp_noise filter = new imp_noise(variance);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Impulsive Noise Added");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Poisson"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,1);
				gd.setVisible(true);
				if (ok)
				{
					poiss_noise filter = new poiss_noise(variance);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Poisson Noise Added");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Uniform"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,1);
				gd.setVisible(true);
				if (ok)
				{
					un_noise filter = new un_noise(variance);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Uniform Noise Added");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("Exponential"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,1);
				gd.setVisible(true);
				if (ok)
				{
					ex_noise filter = new ex_noise(variance);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("Exponential Noise Added");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			else if (label.equals("White Impulsive"))
			{
				eventStart();
				genDialog gd = new genDialog(myframe,1);
				gd.setVisible(true);
				if (ok)
				{
					w_noise filter = new w_noise(variance);
					pic = myframe.createImage(new FilteredImageSource
						(myframe.image.getSource(),filter));
					eventEnd(1);
					AIPP.statusBar.setText("White Impulsive Noise Added");
				}
				else
				{
					eventEnd(0);
					AIPP.statusBar.setText("Operation Canceled");
				}
			}
			myframe.repaint();		
		}
	}

	public void itemStateChanged(ItemEvent e)
	{
		if(e.getSource() instanceof CheckboxMenuItem)
		{
			String label=((CheckboxMenuItem)e.getSource()).getLabel();
			if (label.equals("3x3"))
			{
				myframe.mask5m.setState(false);
				myframe.mask7m.setState(false);
				myframe.mask3m.setState(true);
				masksize=3;
				AIPP.statusBar.setText("Mask Size set to 3x3");
			}
			else if (label.equals("5x5"))
			{
				myframe.mask3m.setState(false);
				myframe.mask7m.setState(false);
				myframe.mask5m.setState(true);
				masksize=5;
				AIPP.statusBar.setText("Mask Size set to 5x5");
			}
			else if (label.equals("7x7"))
			{
				myframe.mask3m.setState(false);
				myframe.mask5m.setState(false);
				myframe.mask7m.setState(true);
				masksize=7;
				AIPP.statusBar.setText("Mask Size set to 7x7");
			}
			else if (label.equals("Custom"))
			{
				custom = myframe.custm.getState();
				AIPP.statusBar.setText("Custom Mask Selected");
			}
			else if(label.equals("Show Image Info"))
			{
				if (((CheckboxMenuItem)e.getSource()).getState() == true)
				{
					myframe.infoPopup = new InfoPopup(myframe);
					myframe.infoPopup.setVisible(true);
					myframe.infoBox = true;
				}
				else
				{
					myframe.infoBox = false;
					myframe.infoPopup.dispose();
				}
			}
		}
		
	}

}
